# frozen_string_literal: true

require 'faraday'
require 'faraday_cdn_metrics/version'
require 'faraday_cdn_metrics/server_caches/fastly'

module FaradayCdnMetrics
  class CdnMetrics < Faraday::Middleware
    # Cache status' shamelessly copied from faraday-http-cache
    CACHE_STATUSES = [
      # The request was not cacheable.
      :unacceptable,

      # The response was cached and can still be used.
      :fresh,

      # The response was cached and the server has validated it with a 304 response.
      :valid,

      # The response was cache but was revalidated by the sserver.
      :invalid,

      # No response was found in the cache.
      :miss,

      # The response can't be cached.
      :uncacheable,

      # The request decided to ignore the cache.
      :bypass
    ].freeze

    EVENT_NAME = 'cdn_metrics.http_cache'

    def initialize(app,
                   instrumenter: nil,
                   instrument_name: EVENT_NAME,
                   server_cache: FaradayCdnMetrics::ServerCaches::Fastly)
      super(app)

      @instrumenter = instrumenter
      @instrument_name = instrument_name
      @server_cache_class = server_cache
    end

    def call(request_env)
      @app.call(request_env).on_complete do |response_env|
        client_cache_status = find_client_cache_status(response_env)
        server_cache_status = find_server_cache_status(response_env)

        @instrumenter.instrument(@instrument_name, {
                                   env: response_env,
                                   client_cache_status: client_cache_status,
                                   server_cache_status: server_cache_status
                                 })
      end
    end

    private

    def find_client_cache_status(env)
      trace = (env[:http_cache_trace] || [])
      CACHE_STATUSES.find { |status| trace.include?(status) } || :unknown
    end

    def find_server_cache_status(env)
      server_cache = @server_cache_class.new(env[:response_headers])

      return :pass if server_cache.pass?
      return :valid if server_cache.hit_while_revalidate?
      return :fresh if server_cache.hit?
      return :miss if server_cache.miss?

      :unknown
    end
  end
end

Faraday::Middleware.register_middleware cdn_metrics: FaradayCdnMetrics::CdnMetrics
